@@ -7,6 +7,7 @@ import sys
 import logging
 from argparse import RawTextHelpFormatter
 import numpy
+from silx.io.url import DataUrl
 from tomoscan.esrf.volume.utils import guess_volumes
 from tomoscan.factory import Factory
 from tomoscan.esrf.volume import (
@@ -26,7 +27,6 @@ from nabu.pipeline.params import files_formats
 from nabu.utils import convert_str_to_tuple
 from nabu.io.cast_volume import _min_max_from_histo
 
-from silx.io.url import DataUrl
 
 _logger = logging.getLogger(__name__)
 
@@ -212,7 +212,9 @@ def main(argv=None):
     elif options.output_type is None:
         raise ValueError("'output_type' or 'output_volume' is expected")
     else:
-        output_volume = get_default_output_volume(input_volume=input_volume, output_type=output_format)
+        output_volume = get_default_output_volume(
+            input_volume=input_volume, output_type=output_format  # pylint: disable=E0606
+        )
     try:
         output_data_type = numpy.dtype(getattr(numpy, options.output_data_type))
     except Exception as e:
