@@ -37,10 +37,15 @@ BootstrapConfig = {
         "default": "",
     },
     "template": {
-        "help": "Use a template configuration file. Available are: id19_pag, id16_holo, id16_ctf. You can also define your own templates via the NABU_TEMPLATES_PATH environment variable.",
+        "help": "Use a template configuration file. Available are: id19_pag, id16_holo, id16_ctf, id16a_fluo, bm05_pag. You can also define your own templates via the NABU_TEMPLATES_PATH environment variable.",
         "default": "",
     },
     "helical": {"help": "Prepare configuration file for helical", "default": 0, "required": False, "type": int},
+    "overwrite": {
+        "help": "Whether to overwrite the output file if exists",
+        "action": "store_const",
+        "const": 1,
+    },
 }
 
 # Default configuration for "zsplit" command
