@@ -1,12 +1,11 @@
-from .. import version
+import sys
 import numpy as np
 import h5py
-import argparse
-import sys
+from .. import version
+from ..utils import DictToObj
+from ..resources.logger import Logger
 from .cli_configs import CreateDistortionMapHorizontallyMatchedFromPolyConfig
 from .utils import parse_params_values
-from ..utils import DictToObj
-from ..resources.logger import Logger, LoggerOrPrint
 
 
 def create_distortion_maps_entry_point(user_args=None):
@@ -52,7 +51,7 @@ def create_distortion_maps_entry_point(user_args=None):
 
     logger = Logger("create_distortion_maps", level=user_args["loglevel"], logfile="create_distortion_maps.log")
 
-    coords_source_x, coords_source_z, new_axis_pos = create_maps_x_and_z(args_dict)
+    coords_source_x, coords_source_z, new_axis_pos = create_maps_x_and_z(args_dict)  # pylint: disable=E0606
 
     with h5py.File(args_dict["target_file"], "w") as f:
         f["coords_source_x"] = coords_source_x
