@@ -1,31 +1,17 @@
-from .. import version
-from os import environ
-
-import argparse
-import shutil
 import os
 import sys
-import re
-import h5py
+from multiprocessing import Pool
 import numpy as np
-
-
-from ..resources.logger import LoggerOrPrint
+from scipy.ndimage import gaussian_filter
+import h5py
+from silx.io.dictdump import h5todict
+from nxtomo.application.nxtomo import NXtomo
+from .. import version
+from ..utils import get_available_threads, DictToObj
+from ..pipeline.estimators import oversample
 from .utils import parse_params_values
 from .cli_configs import DiagToRotConfig
 
-from ..utils import get_available_threads
-
-from nxtomo.application.nxtomo import NXtomo
-
-from multiprocessing import Pool
-
-import h5py
-from nabu.utils import DictToObj
-from nabu.pipeline.estimators import oversample
-from scipy.ndimage import gaussian_filter
-from silx.io.dictdump import h5todict
-
 
 """
   The operations here below rely on diag objects which are found in the result of a nab-helical run with the diag_zpro_run set to a number > 0
@@ -225,12 +211,11 @@ def find_best_interpolating_line(args):
 
             error = (diff_enes / (orig_enes + 1.0e-30)).astype("d").sum()
 
-            if error == error:
-                if not (error > best_error):
-                    best_error = error
-                    best_error_pair = index_ovlp_a, index_ovlp_b
+            if not (error > best_error):
+                best_error = error
+                best_error_pair = index_ovlp_a, index_ovlp_b
 
-    return best_error, best_error_pair
+    return best_error, best_error_pair  # pylint: disable=E0606
 
 
 def main(user_args=None):
@@ -262,7 +247,7 @@ def main(user_args=None):
 
         scan = NXtomo()
         scan.load(file_path=args.original_scan, data_path=args.entry_name)
-        args.near = scan.instrument.detector.estimated_cor_from_motor
+        args.near = scan.instrument.detector.x_rotation_axis_pixel_position
     else:
         pass
 
@@ -305,7 +290,7 @@ def main(user_args=None):
                 """
                 raise ValueError(message)
 
-        arguments_for_multiprocessing.append((diag, overlap_list, args))
+        arguments_for_multiprocessing.append((diag, overlap_list, args))  # pylint: disable=E0606
 
     ncpus = get_available_threads()
     with Pool(processes=ncpus) as pool:
