@@ -1,12 +1,11 @@
 import os
 import numpy as np
 from tomoscan.io import HDF5File
-from silx.third_party.EdfFile import EdfFile
+from fabio.edfimage import EdfImage
 from ..io.utils import get_first_hdf5_entry
 from .utils import parse_params_values
 from .cli_configs import GenerateInfoConfig
 
-
 edf_header_hdf5_path = {
     # EDF Header
     "count_time": "/technique/scan/exposure_time",  # ms in HDF5
@@ -202,7 +201,9 @@ def generate_merged_info_file_content(
     if first_edf_proj is None:
         edf_header = simulate_edf_header(bliss_fname, bliss_entry, return_dict=True)
     else:
-        edf_header = EdfFile(first_edf_proj).GetHeader(0)
+        edf = EdfImage()
+        edf.open(first_edf_proj)
+        edf_header = edf.getHeader()
     # .info File
     if info_file is None:
         info_file_content = simulate_info_file(bliss_fname, bliss_entry, return_dict=True)
@@ -257,3 +258,4 @@ def generate_merged_info_file():
     )
     with open(args["output"], "w") as f:
         f.write(content)
+    return 0
