@@ -49,10 +49,10 @@ def main_helical():
     proc = ProcessConfig(conf_dict=conf_dict, create_logger=logfile)
     logger = proc.logger
 
-    if "rotate_projections" in proc.processing_steps:
+    if "tilt_correction" in proc.processing_steps:
         message = """ The rotate_projections step is activated. The Helical pipelines are not yet suited for  projection rotation
         it will soon be implemented. For the moment
-        you should desactivate the rotation options in nabu.conf
+        you should deactivate the rotation options in nabu.conf
         """
         raise ValueError(message)
 
