@@ -6,12 +6,12 @@ from multiprocessing import cpu_count
 from multiprocessing.pool import ThreadPool
 import numpy as np
 from tomoscan.io import HDF5File
-from tomoscan.esrf.scan.hdf5scan import HDF5TomoScan
+from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
 from ..io.utils import get_first_hdf5_entry
-from ..misc.rotation import Rotation
+from ..processing.rotation import Rotation
 from ..resources.logger import Logger, LoggerOrPrint
 from ..pipeline.config_validators import optional_tuple_of_floats_validator, boolean_validator
-from ..misc.rotation_cuda import CudaRotation, __has_pycuda__
+from ..processing.rotation_cuda import CudaRotation, __has_pycuda__
 from .utils import parse_params_values
 from .cli_configs import RotateRadiosConfig
 
@@ -42,7 +42,7 @@ class HDF5ImagesStackRotation:
         if entry is None or entry == "":
             entry = get_first_hdf5_entry(input_file)
         self.entry = entry
-        self.dataset_info = HDF5TomoScan(input_file, entry=entry)
+        self.dataset_info = NXtomoScan(input_file, entry=entry)
 
     def _get_rotation(self, angle, center):
         if self.use_cuda:
@@ -155,6 +155,7 @@ def rotate_cli():
         use_multiprocessing=use_multiprocessing,
     )
     h5rot.rotate_images()
+    return 0
 
 
 if __name__ == "__main__":
