@@ -49,7 +49,7 @@ class CudaProcessing(ProcessingBase):
         self.stream = stream
         self.device = self.ctx.get_device()
         self.device_name = self.device.name()
-        self.device_id = self.device.get_attribute(dev_attrs.MULTI_GPU_BOARD_GROUP_ID)
+        self.device_id = self.device.get_attribute(dev_attrs.MULTI_GPU_BOARD_GROUP_ID)  # pylint: disable=E0606
 
     def push_context(self):
         self.ctx.push()
@@ -64,7 +64,7 @@ class CudaProcessing(ProcessingBase):
     def kernel(
         self, kernel_name, filename=None, src=None, signature=None, texrefs=None, automation_params=None, **build_kwargs
     ):
-        return CudaKernel(
+        return CudaKernel(  # pylint: disable=E0606
             kernel_name,
             filename=filename,
             src=src,
