@@ -18,12 +18,6 @@ try:
 except ImportError as err:
     __has_pycuda__ = False
     __pycuda_error_msg__ = str(err)
-try:
-    import skcuda
-
-    __has_cufft__ = True
-except ImportError:
-    __has_cufft__ = False
 
 try:
     import cupy
