@@ -1,6 +1,7 @@
 # import math
 import logging
 import numpy as np
+from tqdm import tqdm
 from numpy.polynomial.polynomial import Polynomial
 from silx.math.medianfilter import medfilt2d
 import scipy.fft  # pylint: disable=E0611
@@ -17,18 +18,11 @@ except ImportError:
     plt = None
     __have_matplotlib__ = False
 
-try:
-    from tqdm import tqdm
 
-    def progress_bar(x, verbose=True):
-        if verbose:
-            return tqdm(x)
-        else:
-            return x
-
-except ImportError:
-
-    def progress_bar(x, verbose=True):
+def progress_bar(x, verbose=True):
+    if verbose:
+        return tqdm(x)
+    else:
         return x
 
 
@@ -48,7 +42,6 @@ class AlignmentBase:
         logger=None,
         data_type=np.float32,
         extra_options=None,
-        cor_options=None,
     ):
         """
         Alignment basic functions.
@@ -73,13 +66,6 @@ class AlignmentBase:
 
         self._init_parameters(vert_fft_width, horz_fft_width, verbose, logger, data_type, extra_options=extra_options)
         self._plot_windows = {}
-        self._set_cor_options(cor_options)
-
-    def _set_cor_options(self, cor_options):
-        default_dict = self._default_cor_options.copy()
-        if cor_options is not None:
-            default_dict.update(cor_options)
-        self.cor_options = default_dict
 
     def _init_parameters(self, vert_fft_width, horz_fft_width, verbose, logger, data_type, extra_options=None):
         self.logger = LoggerOrPrint(logger)
