@@ -167,6 +167,11 @@ class DetectorTranslationAlongBeam(AlignmentBase):
         # (according to numpy)
         coeffs_v = Polynomial.fit(img_pos, shifts_vh[:, 0], deg=1).convert().coef
         coeffs_h = Polynomial.fit(img_pos, shifts_vh[:, 1], deg=1).convert().coef
+        # In some cases (singular matrix ?) the output is [0] while in some other its [eps, eps].
+        if len(coeffs_v) == 1:
+            coeffs_v = np.array([coeffs_v[0], coeffs_v[0]])
+        if len(coeffs_h) == 1:
+            coeffs_h = np.array([coeffs_h[0], coeffs_h[0]])
 
         if self.verbose:
             self.logger.info(
