@@ -128,13 +128,7 @@ def estimate_required_memory(
         if process_config.rec_params["method"] == "cone":
             # In cone-beam reconstruction, need both sinograms and reconstruction inside GPU.
             # That's big!
-            # Even more if padding is done at the nabu side.
-            if rec_config["padding_type"] == "zeros":
-                total_memory_needed += 2 * data_volume_size
-            else:
-                total_memory_needed += (
-                    data_volume_size + get_next_power(2 * Nx) * Na * Nz * 4 * 4
-                )  # no idea why the last *4 is necessary
+            total_memory_needed += 2 * data_volume_size
 
     if debug:
         print(
@@ -216,6 +210,7 @@ def estimate_max_chunk_size(
         delta_z = 0
 
     mem = 0
+    # pylint: disable=E0606, E0601
     last_valid_delta_a = delta_a
     last_valid_delta_z = delta_z
     while True:
