@@ -17,7 +17,6 @@ from nxtomo.nxobject.nxdetector import ImageKey
 
 
 class ProcessConfig(ProcessConfigBase):
-
     """
     A ProcessConfig object has these main fields:
       - dataset_info: information about the current dataset
@@ -57,7 +56,7 @@ class ProcessConfig(ProcessConfigBase):
         "flatfield",
         "ccd_correction",
         "double_flatfield",
-        "rotate_projections",
+        "tilt_correction",
         "phase",
         "unsharp_mask",
         "take_log",
@@ -74,13 +73,16 @@ class ProcessConfig(ProcessConfigBase):
         Update the 'dataset_info' (DatasetAnalyzer class instance) data structure with options from user configuration.
         """
         self.logger.debug("Updating dataset information with user configuration")
-        if self.dataset_info.kind == "hdf5":
+        if self.dataset_info.kind == "nx":
             update_dataset_info_flats_darks(
                 self.dataset_info,
                 self.nabu_config["preproc"]["flatfield"],
                 output_dir=self.nabu_config["output"]["location"],
                 darks_flats_dir=self.nabu_config["dataset"]["darks_flats_dir"],
             )
+        elif self.dataset_info.kind == "edf":
+            self.dataset_info.flats = self.dataset_info.get_reduced_flats()
+            self.dataset_info.darks = self.dataset_info.get_reduced_darks()
         self.rec_params = self.nabu_config["reconstruction"]
 
         subsampling_factor, subsampling_start = self.nabu_config["dataset"]["projections_subsampling"]
@@ -361,7 +363,6 @@ class ProcessConfig(ProcessConfigBase):
         method: str or None
             Rotation method: one of the values of `nabu.resources.params.radios_rotation_mode`
         """
-        user_rotate_projections = self.nabu_config["preproc"]["rotate_projections"]
         tilt = self.dataset_info.detector_tilt
         phase_method = self.nabu_config["phase"]["method"]
         do_ctf = phase_method == "CTF"
@@ -369,7 +370,7 @@ class ProcessConfig(ProcessConfigBase):
         do_unsharp = (
             self.nabu_config["phase"]["unsharp_method"] is not None and self.nabu_config["phase"]["unsharp_coeff"] > 0
         )
-        if user_rotate_projections is None and tilt is None:
+        if tilt is None:
             return None
         if do_ctf:
             return "full"
@@ -397,7 +398,6 @@ class ProcessConfig(ProcessConfigBase):
         # For now data is assumed to be on disk (see issue #66).
         tasks.append("read_chunk")
         options["read_chunk"] = {
-            "files": dataset_info.projections,
             "sub_region": None,
             "binning": binning,
             "dataset_subsampling": nabu_config["dataset"]["projections_subsampling"],
@@ -408,7 +408,7 @@ class ProcessConfig(ProcessConfigBase):
         if nabu_config["preproc"]["flatfield"]:
             tasks.append("flatfield")
             options["flatfield"] = {
-                #  ChunkReader handles binning/subsampling by itself,
+                #  Data reader handles binning/subsampling by itself,
                 # but FlatField needs "real" indices (after binning/subsampling)
                 "projs_indices": self.projs_indices(subsampling=False),
                 "binning": binning,
@@ -436,6 +436,9 @@ class ProcessConfig(ProcessConfigBase):
                     "flats_srcurrent": flats_srcurrent,
                 }
             )
+            if len(dataset_info.darks) > 1:
+                self.logger.warning("Cannot do flat-field with more than one reduced dark. Taking the first one.")
+                dataset_info.darks = dataset_info.darks[sorted(dataset_info.darks.keys())[0]]
         #
         # Spikes filter
         #
@@ -460,9 +463,9 @@ class ProcessConfig(ProcessConfigBase):
         # Radios rotation (do it here if possible)
         #
         if self.get_radios_rotation_mode() == "chunk":
-            tasks.append("rotate_projections")
-            options["rotate_projections"] = {
-                "angle": nabu_config["preproc"]["rotate_projections"] or dataset_info.detector_tilt,
+            tasks.append("tilt_correction")
+            options["tilt_correction"] = {
+                "angle": dataset_info.detector_tilt,
                 "center": nabu_config["preproc"]["rotate_projections_center"],
                 "mode": "chunk",
             }
@@ -508,9 +511,9 @@ class ProcessConfig(ProcessConfigBase):
         # Radios rotation (do it here if mode=="full")
         #
         if self.get_radios_rotation_mode() == "full":
-            tasks.append("rotate_projections")
-            options["rotate_projections"] = {
-                "angle": nabu_config["preproc"]["rotate_projections"] or dataset_info.detector_tilt,
+            tasks.append("tilt_correction")
+            options["tilt_correction"] = {
+                "angle": dataset_info.detector_tilt,
                 "center": nabu_config["preproc"]["rotate_projections_center"],
                 "mode": "full",
             }
@@ -555,6 +558,7 @@ class ProcessConfig(ProcessConfigBase):
                 self.rec_params,
                 [
                     "method",
+                    "implementation",
                     "fbp_filter_type",
                     "fbp_filter_cutoff",
                     "padding_type",
@@ -566,8 +570,11 @@ class ProcessConfig(ProcessConfigBase):
                     "end_z",
                     "centered_axis",
                     "clip_outer_circle",
+                    "outer_circle_value",
                     "source_sample_dist",
                     "sample_detector_dist",
+                    "hbp_legs",
+                    "hbp_reduction_steps",
                 ],
             )
             rec_options = options["reconstruction"]
@@ -586,6 +593,8 @@ class ProcessConfig(ProcessConfigBase):
                 voxel_size,
             )  # pix size is in microns in dataset_info
 
+            rec_options["iterations"] = nabu_config["reconstruction"]["iterations"]
+
             # x/y/z position information
             def get_mean_pos(position_array):
                 if position_array is None:
