@@ -146,7 +146,9 @@ class FullFieldReconstructor:
             else:
                 self.gpu_mem = self.resources["gpus"][self._gpu_id]["memory_GB"] * self.gpu_mem_fraction
         if backend == "cuda":
-            self._pipeline_cls = CudaChunkedPipeline
+            if not (__has_pycuda__):
+                raise RuntimeError("pycuda not avilable")
+            self._pipeline_cls = CudaChunkedPipeline  # pylint: disable=E0606
         self.backend = backend
 
     def _compute_max_chunk_size(self):
@@ -251,7 +253,7 @@ class FullFieldReconstructor:
         if self.process_config.processing_options.get("phase", {}).get("method", None) == "CTF":
             force_grouped_mode = True
             msg = "CTF phase retrieval needs to process full radios"
-        if self.process_config.processing_options.get("rotate_projections", {}).get("angle", 0) > 15:
+        if (self.process_config.dataset_info.detector_tilt or 0) > 15:
             force_grouped_mode = True
             msg = "Radios rotation with a large angle needs to process full radios"
         if self.process_config.resume_from_step == "sinogram" and force_grouped_mode:
@@ -263,7 +265,7 @@ class FullFieldReconstructor:
             self.chunk_shape = (self.n_angles, self.chunk_size, self.n_x)
         else:
             # Fall-back mode (slower)
-            self.logger.warning(msg)
+            self.logger.warning(msg)  # pylint: disable=E0606
             self._pipeline_mode = "grouped"
             self._compute_max_group_size()
             self.chunk_shape = (self.group_size, self.delta_z, self.n_x)
@@ -286,9 +288,10 @@ class FullFieldReconstructor:
             phase_margin = self._compute_phase_margin()
             translations_margin = self._compute_translations_margin()
             cone_margin = self._compute_cone_overlap()
+            rot_margin = self._compute_rotation_margin()
             # TODO radios rotation/movements
-            margin_v = max(unsharp_margin[0], phase_margin[0], translations_margin[0], cone_margin[0])
-            margin_h = max(unsharp_margin[1], phase_margin[1], translations_margin[1], cone_margin[1])
+            margin_v = max(unsharp_margin[0], phase_margin[0], translations_margin[0], cone_margin[0], rot_margin[0])
+            margin_h = max(unsharp_margin[1], phase_margin[1], translations_margin[1], cone_margin[1], rot_margin[1])
             if margin_v > 0:
                 self.logger.info("Estimated margin: %d pixels" % margin_v)
 
@@ -360,7 +363,7 @@ class FullFieldReconstructor:
         return (max_overlap, 0)
 
     def _compute_rotation_margin(self):
-        if "rotate_projections" in self.process_config.processing_steps:
+        if "tilt_correction" in self.process_config.processing_steps:
             # Partial radios rotation yields too much error in single-slice mode
             # Forcing a big margin circumvents the problem
             # This is likely to trigger the 'grouped mode', but perhaps grouped mode should always be used when rotating radios
@@ -505,8 +508,8 @@ class FullFieldReconstructor:
 
     def _print_tasks(self):
         for task in self.tasks:
-            margin_up, margin_down = task["phase_margin"][0]
-            s_u, s_d = task["sub_region"]
+            margin_up, margin_down = task["margin"][0]
+            s_u, s_d = task["sub_region"][1]
             print(
                 "Top Margin: [%04d, %04d[  |  Slices: [%04d, %04d[  |  Bottom Margin: [%04d, %04d["  # pylint: disable=E1307
                 % (s_u, s_u + margin_up, s_u + margin_up, s_d - margin_down, s_d - margin_down, s_d)
@@ -713,12 +716,13 @@ class FullFieldReconstructor:
         # (these values are checked in ProcessConfig._configure_resume())
         #
         patched_start_end_z = False
-        user_rec_config = self.process_config.processing_options["reconstruction"]
         if (
             self._margin_v > 0
             and process_name != "reconstruction"
             and self.process_config.is_before_radios_cropping(process_name)
+            and "reconstruction" in self.process_config.processing_steps
         ):
+            user_rec_config = self.process_config.processing_options["reconstruction"]
             patched_start_end_z = True
             old_start_z = user_rec_config["start_z"]
             old_end_z = user_rec_config["end_z"]
