@@ -5,6 +5,8 @@ import math
 
 
 class GriddedAccumulator:
+    nominal_current = 0.2
+
     def __init__(
         self,
         gridded_radios,
@@ -12,11 +14,16 @@ class GriddedAccumulator:
         diagnostic_radios,
         diagnostic_weights,
         diagnostic_angles,
+        diagnostic_searched_angles_rad_clipped,
+        diagnostic_zpix_transl,
+        diag_zpro_run=0,
         dark=None,
         flat_indexes=None,
         flats=None,
         weights=None,
         double_flat=None,
+        radios_srcurrent=None,
+        flats_srcurrent=None,
     ):
         """
         This class creates, for a selected volume slab, a standard set of radios from an helical dataset.
@@ -33,22 +40,34 @@ class GriddedAccumulator:
         gridded_weights : 3d np.array
            same shape as gridded_radios, but it will accumulate the weights, during calls to
            extract_preprocess_with_flats
-        diagnostic_radios   : 3d np.array,  a stack composed of two radios
+        diag_zpro_run: int
+           if > 0 then only the diagnostics are filled, and no accumulation is done
+
+        diagnostic_searched_angles_rad_clipped:
+           the angles between 0 and 2pi. The contributions to diagnostic will be searched for these angles
+           plus for the same angles + 2pi ( following turn)
+
+        diagnostic_radios   : 3d np.array,  a stack composed of
            each radio must have the same size as a radio of the gridded_radios argument.
            During the calls to extract_preprocess_with_flats methods,
-           the first radio will collect the transformed data for angle=0 ( and the neighbouring ones
-           according to angular interpolation coefficients) and this only for the first occurring turn.
-           The second radio will be initialised  at the second turn, if any. These array are meant to be used
-           to  check the translation step over one turn.
+           the  radios will collect the transformed data for the angles given by diagnostic_searched_angles_rad_clipped and redundancy
         diagnostic_weights: 3d np.array  a stack composed of two radios
            Same shape as diagnostic_radios. The weigths for diagnostic radios ( will be zero on pixel
            where no data is available, or where the weight is null)
         diagnostic_angles : 1D np.array
-           Must have shape==(2,). The two entries will be filled with the angles at which the contributions
+           Must have shape==(2*len(diagnostic_searched_angles_rad_clipped),). The entries will be filled with the angles at which the contributions
            to diagnostic_radios have been summed.
-        dark =  None or 2D np.array
+        diagnostic_zpix_transl: 1D np.array
+           same as for diagnostic_angles, but for vertical translation in pixels
+        dark:  None or 2D np.array
            must have the shape of the detector ( generally larger that a radio of gridded_radios)
            If given, the dark will be subtracted from data and flats.
+        radios_srcurrent: 1D np.array
+             the machine current for every radio
+        flats_srcurrent: 1D np.array
+             the machine current for every flat
+
+
         flat_indexes: None or a list of integers
            the projection index corresponding to the flats
         flats   : None or 3D np.array
@@ -61,12 +80,22 @@ class GriddedAccumulator:
            If given, the double flat will be applied (division by double_flat)
            Must have  the same shape as the detector.
         """
+        self.diag_zpro_run = diag_zpro_run
         self.gridded_radios = gridded_radios
         self.gridded_weights = gridded_weights
         self.diagnostic_radios = diagnostic_radios
         self.diagnostic_weights = diagnostic_weights
         self.diagnostic_angles = diagnostic_angles
+
+        self.diagnostic_zpix_transl = diagnostic_zpix_transl
+        self.diagnostic_searched_angles_rad_clipped = diagnostic_searched_angles_rad_clipped
+
         self.dark = dark
+
+        self.radios_srcurrent = radios_srcurrent
+
+        self.flats_srcurrent = flats_srcurrent
+
         self.flat_indexes = flat_indexes
         self.flat_indexes_reverse_map = dict(
             [(global_index, local_index) for (local_index, global_index) in enumerate(flat_indexes)]
@@ -76,7 +105,14 @@ class GriddedAccumulator:
         self.double_flat = double_flat
 
     def extract_preprocess_with_flats(
-        self, subchunk_slice, subchunk_file_indexes, chunk_info, subr_start_end, dtasrc_start_end, data_raw
+        self,
+        subchunk_slice,
+        subchunk_file_indexes,
+        chunk_info,
+        subr_start_end,
+        dtasrc_start_end,
+        data_raw,
+        radios_angular_range_slicing,
     ):
         """
         This functions is meant to be called providing, each time, a subset of the data
@@ -120,6 +156,13 @@ class GriddedAccumulator:
             They indicated, vertically, the detector portion the data_raw data correspond to
         data_raw: np.array 3D
             the data which correspond to a limited detector stripe and a limited angular subset
+        radios_angular_range_slicing:
+            my_subsampled_indexes  is important in order to compare the
+            radios positions with respect to the flat position, and these position
+            are given as the sequential acquisition number which counts everything ( flats, darks, radios )
+            Insteqd, in order to access array which spans only the radios, we need to have an idea of where we are.
+            this is provided by radios_angular_range_slicing which addresses the radios domain
+
         """
 
         # the object below is going to containing some auxiliary variable that are use to reframe the data.
@@ -141,7 +184,9 @@ class GriddedAccumulator:
         )
 
         # extraction of the data
-        self._extract_preprocess_with_flats(data_raw, reframing_infos, chunk_info, radios_subset)
+        self._extract_preprocess_with_flats(
+            data_raw, reframing_infos, chunk_info, radios_subset, radios_angular_range_slicing
+        )
 
         if self.weights is not None:
             # ... and, if required, extraction of the associated weights
@@ -156,7 +201,9 @@ class GriddedAccumulator:
         # and the remaining part is a simple projection over the accumulators,  for
         # the data and for the weights
         my_angles = chunk_info.angles_rad[subchunk_slice]
+
         n_gridded_angles = self.gridded_radios.shape[0]
+
         my_i_float = my_angles * (n_gridded_angles / (2 * math.pi))
 
         tmp_i_rounded = np.floor(my_i_float).astype(np.int32)
@@ -164,17 +211,52 @@ class GriddedAccumulator:
         my_i0 = np.mod(tmp_i_rounded, n_gridded_angles)
         my_i1 = np.mod(my_i0 + 1, n_gridded_angles)
 
-        for i0, epsilon, i1, data, weight, original_angle in zip(
-            my_i0, my_epsilon, my_i1, radios_subset, radios_weights_subset, chunk_info.angles_rad[subchunk_slice]
+        if self.diag_zpro_run:
+            # these are used only when collection the diagnostics
+
+            # an estimation of the angular step
+            my_angle_step_rad = abs(np.diff(chunk_info.angles_rad[subchunk_slice]).mean())
+            my_angles_02pi = np.mod(my_angles, 2 * np.pi)
+            # bins are delimited by ticks
+            ticks = np.empty(2 * len(self.diagnostic_searched_angles_rad_clipped), "f")
+            ticks[::2] = self.diagnostic_searched_angles_rad_clipped - my_angle_step_rad / 2
+            ticks[1::2] = ticks[::2] + my_angle_step_rad
+
+        for i0, epsilon, i1, data, weight, original_angle, original_zpix_transl in zip(
+            my_i0,
+            my_epsilon,
+            my_i1,
+            radios_subset,
+            radios_weights_subset,
+            chunk_info.angles_rad[subchunk_slice],
+            chunk_info.z_pix_per_proj[subchunk_slice],
         ):
             data_token = data * weight
-            self.gridded_radios[i0] += data_token * (1 - epsilon)
-            self.gridded_radios[i1] += data_token * epsilon
 
-            self.gridded_weights[i0] += weight * (1 - epsilon)
-            self.gridded_weights[i1] += weight * epsilon
+            if not self.diag_zpro_run:
+                self.gridded_radios[i0] += data_token * (1 - epsilon)
+                self.gridded_radios[i1] += data_token * epsilon
+
+                self.gridded_weights[i0] += weight * (1 - epsilon)
+                self.gridded_weights[i1] += weight * epsilon
+
+        # building the intervals around the diagnostic angles
+
+        if self.diag_zpro_run:
+            my_i0 = np.searchsorted(ticks, my_angles_02pi)
+
+            for i0, a02pi, data, weight, original_angle, original_zpix_transl in zip(
+                my_i0,
+                my_angles_02pi,
+                radios_subset,
+                radios_weights_subset,
+                chunk_info.angles_rad[subchunk_slice],
+                chunk_info.z_pix_per_proj[subchunk_slice],
+            ):
+                if i0 % 2 == 0:
+                    # not in an intervals
+                    continue
 
-            if i0 == 0 or i1 == 0:
                 # There is a contribution to the first regridded radio ( the one indexed by 0)
                 # We build  two diagnostics for the contributions to this radio.
                 # The first for the first pass (i_diag=0)
@@ -183,25 +265,49 @@ class GriddedAccumulator:
                 # To discriminate we introduce
                 # An angular margin beyond which we know that a possible contribution
                 # is coming from another turn
-                safe_angular_margin = 3.14 / 10
-                for i_diag in range(2):
-                    if original_angle < self.diagnostic_angles[i_diag] + safe_angular_margin:
+
+                safe_angular_margin = 3.14 / 40
+
+                my_dist = abs(
+                    a02pi - self.diagnostic_searched_angles_rad_clipped[(i0 - 1) // 2 : (i0 - 1) // 2 + 1].mean()
+                )
+
+                # print(" i0 ", i0, " original_angle ",  original_angle, "  a02pi" , a02pi, " my_dist " , my_dist)
+
+                # consider fist pass and second possible pass. There might be further passes which we dont consider here
+                i_diag_list = [(i0 - 1) // 2, (i0 - 1) // 2 + len(self.diagnostic_searched_angles_rad_clipped)]
+                for i_redundancy, i_diag in enumerate(i_diag_list):
+                    # print("IRED ", i_redundancy)
+                    if i_redundancy:
+                        # to avoid, in z_stages with >360 range for one single stage, to fill the second items which should instead be filled by another stage.
+                        if abs(original_zpix_transl - self.diagnostic_zpix_transl[i_diag_list[0]]) < 2.0:
+                            # print( " >>>>>> stesso z" , i_redundancy )
+                            continue
+
+                    if np.isnan(self.diagnostic_angles[i_diag]) or (
+                        abs(original_angle) < abs(self.diagnostic_angles[i_diag] + safe_angular_margin)
+                    ):
                         # we are searching for the first contributions ( the one at the lowest angle)
                         # for the two diagnostics. With the constraint that the second is at an higher angle
                         # than the first. So if we are here this means that we have found an occurrence with
                         # lower angle and we discard what we could have previously found.
+
                         self.diagnostic_radios[i_diag][:] = 0
                         self.diagnostic_weights[i_diag][:] = 0
                         self.diagnostic_angles[i_diag] = original_angle
 
-                    if abs(original_angle - self.diagnostic_angles[i_diag]) < safe_angular_margin:
-                        if i0 == 0:
-                            factor = 1 - epsilon
-                        else:
-                            factor = epsilon
+                        self.diagnostic_zpix_transl[i_diag] = original_zpix_transl
+                    else:
+                        continue
+
+                    if abs(my_dist) <= my_angle_step_rad * 1.1:
+                        factor = 1 - abs(my_dist) / (my_angle_step_rad)
+
                         self.diagnostic_radios[i_diag] += data_token * factor
                         self.diagnostic_weights[i_diag] += weight * factor
                         break
+                    else:
+                        pass
 
     class _ReframingInfos:
         def __init__(self, chunk_info, subchunk_slice, subr_start_end, dtasrc_start_end, subchunk_file_indexes):
@@ -220,7 +326,9 @@ class GriddedAccumulator:
 
             self.floating_subregion = None, None, floating_start_z, floating_end_z
 
-    def _extract_preprocess_with_flats(self, data_raw, reframing_infos, chunk_info, output, it_is_weight=False):
+    def _extract_preprocess_with_flats(
+        self, data_raw, reframing_infos, chunk_info, output, radios_angular_range_slicing=None, it_is_weight=False
+    ):
         if not it_is_weight:
             if self.dark is not None:
                 data_raw = data_raw - self.dark[reframing_infos.dtasrc_start_z : reframing_infos.dtasrc_end_z]
@@ -231,7 +339,12 @@ class GriddedAccumulator:
                     if self.dark is not None:
                         flat = flat - self.dark[reframing_infos.dtasrc_start_z : reframing_infos.dtasrc_end_z]
 
-                    data_raw[i] = data_raw[i] / flat
+                    if self.radios_srcurrent is not None:
+                        factor = self.nominal_current / self.radios_srcurrent[radios_angular_range_slicing.start + i]
+                    else:
+                        factor = 1
+
+                    data_raw[i] = data_raw[i] * factor / flat
 
             if self.double_flat is not None:
                 data_raw = data_raw / self.double_flat[reframing_infos.dtasrc_start_z : reframing_infos.dtasrc_end_z]
@@ -271,12 +384,35 @@ class GriddedAccumulator:
             flat_data = self.flats[self.flat_indexes_reverse_map[prev_next[0]]][slice_y, slice_x]
         else:  # interpolate
             prev_idx, next_idx = prev_next
-            flat_data_prev = self.flats[self.flat_indexes_reverse_map[prev_idx]][slice_y, slice_x]
-            flat_data_next = self.flats[self.flat_indexes_reverse_map[next_idx]][slice_y, slice_x]
+
+            n_prev = self.flat_indexes_reverse_map[prev_idx]
+            n_next = self.flat_indexes_reverse_map[next_idx]
+
+            flat_data_prev = self.flats[n_prev][slice_y, slice_x]
+            flat_data_next = self.flats[n_next][slice_y, slice_x]
+
+            if self.flats_srcurrent is not None:
+                prev_current_factor = self.nominal_current / self.flats_srcurrent[n_prev]
+                next_current_factor = self.nominal_current / self.flats_srcurrent[n_next]
+            else:
+                prev_current_factor = 1
+                next_current_factor = 1
+
             delta = next_idx - prev_idx
             w1 = 1 - (idx - prev_idx) / delta
             w2 = 1 - (next_idx - idx) / delta
-            flat_data = w1 * flat_data_prev + w2 * flat_data_next
+
+            if self.dark is not None:
+                dark = self.dark[slice_y, slice_x]
+            else:
+                dark = 0
+
+            flat_data = (
+                dark
+                + w1 * (flat_data_prev - dark) * prev_current_factor
+                + w2 * (flat_data_next - dark) * next_current_factor
+            )
+
         if flat_data.dtype != dtype:
             flat_data = np.ascontiguousarray(flat_data, dtype=dtype)
         return flat_data
@@ -364,7 +500,7 @@ def padding_logic(subr_start_z, subr_end_z, dtasrc_start_z, dtasrc_end_z):
 
 
 def get_reconstruction_space(span_info, min_scanwise_z, end_scanwise_z, phase_margin_pix):
-    """Utility function which, given the span_info object, creates the auxiliary collection arrays
+    """Utility function, so far used only by the unit test, which, given the span_info object, creates the auxiliary collection arrays
     and initialises the  my_z_min, my_z_end variable keeping into account the scan direction
     and the min_scanwise_z, end_scanwise_z input arguments
     Parameters
@@ -404,9 +540,17 @@ def get_reconstruction_space(span_info, min_scanwise_z, end_scanwise_z, phase_ma
     # the accumulators
     gridded_radios = np.zeros([n_gridded_angles, radios_h, span_info.detector_shape_vh[1]], np.float32)
     gridded_cumulated_weights = np.zeros([n_gridded_angles, radios_h, span_info.detector_shape_vh[1]], np.float32)
-    diagnostic_radios = np.zeros((2,) + gridded_radios.shape[1:], np.float32)
-    diagnostic_weights = np.zeros((2,) + gridded_radios.shape[1:], np.float32)
-    diagnostic_proj_angle = np.zeros([2], "f")
+
+    # this utility function is meant for testing the reconstruction only, not the diagnostic collection.
+    # However we build diagnostic targets all the same to feed something through the API
+    # which contemplates the diagnostics. So that the unit test runs correctly
+
+    diagnostic_radios = np.zeros((4,) + gridded_radios.shape[1:], np.float32)
+    diagnostic_weights = np.zeros((4,) + gridded_radios.shape[1:], np.float32)
+    diagnostic_proj_angle = np.zeros([4], "f")
+
+    diagnostic_searched_angles_rad_clipped = (0.5 + np.arange(2)) * (2 * np.pi / (2))
+    diagnostic_zpix_transl = np.zeros([4], "f")
 
     gridded_angles_rad = np.arange(n_gridded_angles) * 2 * np.pi / n_gridded_angles
     gridded_angles_deg = np.rad2deg(gridded_angles_rad)
@@ -422,6 +566,8 @@ def get_reconstruction_space(span_info, min_scanwise_z, end_scanwise_z, phase_ma
             "diagnostic_radios": diagnostic_radios,
             "diagnostic_weights": diagnostic_weights,
             "diagnostic_proj_angle": diagnostic_proj_angle,
+            "diagnostic_searched_angles_rad_clipped": diagnostic_searched_angles_rad_clipped,
+            "diagnostic_zpix_transl": diagnostic_zpix_transl,
             "gridded_angles_rad": gridded_angles_rad,
             "gridded_angles_deg": gridded_angles_deg,
         },
