@@ -1008,7 +1008,7 @@ class HelicalChunkedRegriddedPipeline:
             np.array((subr_start_z, subr_end_z), "i"),
             np.array((dtasrc_start_z, dtasrc_end_z), "i"),
             data_raw,
-            radios_angular_range_slicing  # my_subsampled_indexes  is important in order to compare the
+            radios_angular_range_slicing,  # my_subsampled_indexes  is important in order to compare the
             # radios positions with respect to the flat position, and these position
             # are given as the sequential acquisition number which counts everything ( flats, darks, radios )
             # Insteqd, in order to access array which spans only the radios, we need to have an idea of where we are.
