@@ -468,7 +468,7 @@ class HelicalReconstructorRegridded:
             phase_margin=task["phase_margin"],
             reading_granularity=self.reading_granularity,
             span_info=self._span_info,
-            diag_zpro_run=self.diag_zpro_run
+            diag_zpro_run=self.diag_zpro_run,
             # cuda_options=self.cuda_options
         )
 
