@@ -186,7 +186,7 @@ class SpanStrategy:
 
     def get_informative_string(self):
         doable_span_v = self.get_doable_span()
-        if self.z_pix_per_proj[-1] > self.z_pix_per_proj[-1]:
+        if self.z_pix_per_proj[-1] >= self.z_pix_per_proj[-1]:
             direction = "ascending"
         else:
             direction = "descending"
