@@ -33,7 +33,18 @@ padding_modes = {
     "zero": "zeros",
 }
 
-reconstruction_methods = {"fbp": "FBP", "cone": "cone", "conic": "cone", "none": None, "": None}
+reconstruction_methods = {
+    "fbp": "FBP",
+    "cone": "cone",
+    "conic": "cone",
+    "none": None,
+    "": None,
+    "mlem": "mlem",
+    "fluo": "mlem",
+    "em": "mlem",
+    "hbp": "HBP",
+    "ghbp": "HBP",
+}
 
 fbp_filters = {
     "ramlak": "ramlak",
@@ -64,6 +75,14 @@ optim_algorithms = {
     "fista": "fista",
 }
 
+reco_implementations = {
+    "astra": "astra",
+    "corrct": "corrct",
+    "corr-ct": "corrct",
+    "nabu": "nabu",
+    "": None,
+}
+
 files_formats = {
     "h5": "hdf5",
     "hdf5": "hdf5",
@@ -123,6 +142,7 @@ cor_methods = {
     "fourier-angle": "fourier-angles",
     "fourier angle": "fourier-angles",
     "octave-accurate": "octave-accurate",
+    "vo": "vo",
 }
 
 
