@@ -28,7 +28,7 @@ def pipeline_step(step_attr, step_desc):
     def decorator(func):
         def wrapper(*args, **kwargs):
             self = args[0]
-            if self.__getattribute__(step_attr) is None:
+            if getattr(self, step_attr, None) is None:
                 return
             self.logger.info(step_desc)
             res = func(*args, **kwargs)
@@ -38,7 +38,9 @@ def pipeline_step(step_attr, step_desc):
                 for callback in callbacks:
                     callback(self)
             if self.datadump_manager is not None and step_name in self.datadump_manager.data_dump:
-                self.datadump_manager.dump_data_to_file(step_name, self.radios)
+                self.datadump_manager.dump_data_to_file(
+                    step_name, self.radios, crop_margin=not (self._radios_were_cropped)
+                )
             return res
 
         return wrapper
