@@ -1,5 +1,6 @@
 from os import path
 from tomoscan.esrf import TIFFVolume, MultiTIFFVolume, EDFVolume, JP2KVolume
+from tomoscan.esrf.volume.singleframebase import VolumeSingleFrameBase
 from ..utils import check_supported, get_num_threads
 from ..resources.logger import LoggerOrPrint
 from ..io.writer import NXProcessWriter, HSTVolVolume, NXVolVolume
@@ -8,7 +9,6 @@ from .params import files_formats
 
 
 class WriterManager:
-
     """
     This class is a wrapper on top of all "writers".
     It will create the right "writer" with all the necessary options, and the histogram writer.
@@ -114,6 +114,7 @@ class WriterManager:
         return vol_writer.data_url.file_path()
 
     def _init_writer(self):
+        self._writer_was_already_initialized = self.extra_options.get("writer_initialized", False)
         if self.file_format in ["tiff", "edf", "jp2", "hdf5"]:
             writer_kwargs = {
                 "folder": self.output_dir,
@@ -144,6 +145,11 @@ class WriterManager:
         self._h5_entry = self.metadata.get("entry", "entry")
         self.writer = self._writer_classes[self.file_format](**writer_kwargs)
         self.fname = self.get_fname(self.writer)
+        # In certain cases, tomoscan needs to remove any previous existing volume filess
+        # and avoid calling 'clean_output_data' when writing downstream (for chunk processing)
+        if isinstance(self.writer, VolumeSingleFrameBase):
+            self.writer.skip_existing_data_files_removal = self._writer_was_already_initialized
+        # ---
         if path.exists(self.fname):
             err = "File already exists: %s" % self.fname
             if self.overwrite:
@@ -188,7 +194,9 @@ class WriterManager:
         self.writer.metadata = self.metadata
         self.writer.save_metadata()
 
-    def write_data(self, data):
+    def write_data(self, data, metadata=None):
         self.writer.data = data
+        if metadata is not None:
+            self.writer.metadata = metadata
         self.writer.save()
         # self._write_metadata()
