@@ -2,7 +2,7 @@ from multiprocessing.pool import ThreadPool
 from bisect import bisect_left
 import numpy as np
 from ..io.reader import load_images_from_dataurl_dict
-from ..utils import check_supported, get_num_threads
+from ..utils import check_supported, deprecated_class, get_num_threads
 
 
 class FlatFieldArrays:
@@ -228,6 +228,7 @@ class FlatFieldArrays:
                 f_idx, weights = _interp_nearest(idx, prev_next)
             elif self.interpolation == "linear":
                 f_idx, weights = _interp_linear(idx, prev_next)
+            # pylint: disable=E0606
             self.flats_idx[i] = f_idx
             self.flats_weights[i] = weights
 
@@ -376,6 +377,9 @@ class FlatFieldArrays:
 FlatField = FlatFieldArrays
 
 
+@deprecated_class(
+    "FlatFieldDataUrls is deprecated since 2024.2.0 and will be removed in a future version", do_print=True
+)
 class FlatFieldDataUrls(FlatField):
     def __init__(
         self,
