@@ -18,11 +18,10 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
         delta_beta=250.0,
         pixel_size=1e-6,
         padding="edge",
-        margin=None,
         cuda_options=None,
         fftw_num_threads=None,  # COMPAT.
         fft_num_threads=None,
-        fft_backend="skcuda",
+        fft_backend="vkfft",
     ):
         """
         Please refer to the documentation of
@@ -37,7 +36,6 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
             delta_beta=delta_beta,
             pixel_size=pixel_size,
             padding=padding,
-            margin=margin,
             use_rfft=True,
             fft_num_threads=False,
         )
@@ -118,14 +116,13 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
             raise ValueError("Expected either numpy array, pycuda array or pycuda buffer")
 
     def get_output(self, output):
-        s0, s1 = self.shape_inner
-        ((U, _), (L, _)) = self.margin
+        s0, s1 = self.shape
         if output is None:
             # copy D2H
-            return self.d_radio_padded[U : U + s0, L : L + s1].get()
-        assert output.shape == self.shape_inner
+            return self.d_radio_padded[:s0, :s1].get()
+        assert output.shape == self.shape
         assert output.dtype == np.float32
-        output[:, :] = self.d_radio_padded[U : U + s0, L : L + s1]
+        output[:, :] = self.d_radio_padded[:s0, :s1]
         return output
 
     def apply_filter(self, radio, output=None):
