@@ -4,13 +4,14 @@ import tempfile
 import numpy as np
 import pytest
 from silx.io.url import DataUrl
-from tomoscan.esrf.mock import MockHDF5
+from tomoscan.esrf.mock import MockNXtomo
 from nabu.io.reader import HDF5Reader
 from nabu.preproc.double_flatfield import DoubleFlatField
-from nabu.preproc.double_flatfield_cuda import CudaDoubleFlatField, __has_pycuda__
+from nabu.cuda.utils import __has_pycuda__, get_cuda_context
 
 if __has_pycuda__:
     import pycuda.gpuarray as garray
+    from nabu.preproc.double_flatfield_cuda import CudaDoubleFlatField, __has_pycuda__
 
 
 @pytest.fixture(scope="class")
@@ -19,7 +20,7 @@ def bootstrap(request):
     cls.tmpdir = tempfile.TemporaryDirectory()
     dname = cls.tmpdir.name
     cls.dname = dname
-    radios = MockHDF5(
+    radios = MockNXtomo(
         path.join(dname, "tmp"),
         10,
         n_ini_proj=10,
@@ -47,6 +48,14 @@ def bootstrap(request):
     cls.golden = golden / 10
     cls.tol = 1e-4
 
+    if __has_pycuda__:
+        cls.ctx = get_cuda_context(cleanup_at_exit=False)
+
+    yield
+
+    if __has_pycuda__:
+        cls.ctx.pop()
+
 
 @pytest.mark.usefixtures("bootstrap")
 class TestDoubleFlatField:
@@ -64,13 +73,13 @@ class TestDoubleFlatField:
 
     @pytest.mark.skipif(not (__has_pycuda__), reason="Need pycuda for double flatfield with cuda backend")
     def test_dff_cuda(self):
-        dff = CudaDoubleFlatField(self.radios.shape, result_url=self.ff_cuda_dump_url)
+        dff = CudaDoubleFlatField(self.radios.shape, result_url=self.ff_cuda_dump_url, cuda_options={"ctx": self.ctx})
         d_radios = garray.to_gpu(self.radios)
         mydf = dff.get_double_flatfield(radios=d_radios).get()
 
         assert path.isfile(dff.result_url.file_path())
 
-        dff2 = CudaDoubleFlatField(self.radios.shape, result_url=self.ff_cuda_dump_url)
+        dff2 = CudaDoubleFlatField(self.radios.shape, result_url=self.ff_cuda_dump_url, cuda_options={"ctx": self.ctx})
         mydf2 = dff2.get_double_flatfield(radios=d_radios).get()
 
         assert np.max(np.abs(mydf2 - mydf)) < self.tol
