@@ -4,7 +4,6 @@ from ..cuda.processing import CudaProcessing, __has_pycuda__
 from ..processing.padding_cuda import CudaPadding
 from ..processing.fft_cuda import get_fft_class, get_available_fft_implems
 from ..processing.transpose import CudaTranspose
-from ..thirdparty.tomocupy_remove_stripe import remove_all_stripe_pycuda, __have_tomocupy_deringer__
 from .rings import MunchDeringer, SinoMeanDeringer, VoDeringer
 
 if __has_pycuda__:
@@ -18,6 +17,8 @@ try:
 except ImportError:
     __have_pycudwt__ = False
 
+# pylint: disable=E0606
+
 
 class CudaMunchDeringer(MunchDeringer):
     def __init__(
@@ -28,7 +29,7 @@ class CudaMunchDeringer(MunchDeringer):
         wname="db15",
         padding=None,
         padding_mode="edge",
-        fft_backend="skcuda",
+        fft_backend="vkfft",
         cuda_options=None,
     ):
         """
@@ -63,8 +64,8 @@ class CudaMunchDeringer(MunchDeringer):
         self._setup_fw_kernel()
 
     def _check_can_use_wavelets(self):
-        if not (__have_pycudwt__):
-            raise ValueError("Needs pycudwt to use this class")
+        if not (__have_pycudwt__ and __has_pycuda__):
+            raise ValueError("Needs pycuda and pycudwt to use this class")
 
     def _init_padding(self):
         if self.padding is None:
@@ -220,20 +221,22 @@ class CudaVoDeringer(VoDeringer):
     """
 
     def _check_requirement(self):
+        # Do it here, otherwise cupy shows warnings at import even if not used
+        from ..thirdparty.tomocupy_remove_stripe import remove_all_stripe_pycuda, __have_tomocupy_deringer__
+
         if not (__have_tomocupy_deringer__):
             raise ImportError("need cupy")
+        self._remove_all_stripe_pycuda = remove_all_stripe_pycuda
 
     def remove_rings_radios(self, radios):
-        return remove_all_stripe_pycuda(radios, **self._remove_all_stripe_kwargs)
+        return self._remove_all_stripe_pycuda(radios, layout="radios", **self._remove_all_stripe_kwargs)
 
     def remove_rings_sinograms(self, sinos):
-        radios = sinos.transpose(axes=(1, 0, 2))  # view, no copy
-        self.remove_rings_radios(radios)
-        return sinos
+        return self._remove_all_stripe_pycuda(sinos, layout="sinos", **self._remove_all_stripe_kwargs)
 
     def remove_rings_sinogram(self, sino):
-        radios = sino.reshape(sino.shape[0], 1, sino.shape[1])  # no copy
-        self.remove_rings_radios(radios)
+        sinos = sino.reshape((1, sino.shape[0], -1))  # no copy
+        self.remove_rings_sinograms(sinos)
         return sino
 
     remove_rings = remove_rings_sinograms
@@ -261,10 +264,9 @@ class CudaSinoMeanDeringer(SinoMeanDeringer):
             filename=get_cuda_srcfile("normalization.cu"),
             signature="PPiii",
         )
-        self._mean_kernel_block = (32, 1, 1)
-        self._mean_kernel_grid = [updiv(self.sinos_shape[-1], self._mean_kernel_block[0]), 1, 1]
-        self._mean_kernel_args = [self.d_sino_profile, np.int32(self.n_x), np.int32(self.n_angles), np.int32(1)]
-
+        self._mean_kernel_block = (32, 1, 32)
+        self._mean_kernel_grid = [updiv(a, b) for a, b in zip(self.sinos_shape[::-1], self._mean_kernel_block)]
+        self._mean_kernel_args = [self.d_sino_profile, np.int32(self.n_x), np.int32(self.n_angles), np.int32(self.n_z)]
         self._mean_kernel_kwargs = {
             "grid": self._mean_kernel_grid,
             "block": self._mean_kernel_block,
@@ -276,11 +278,9 @@ class CudaSinoMeanDeringer(SinoMeanDeringer):
             signature="PPiii",
             options=["-DGENERIC_OP=%d" % (3 if self.mode == "divide" else 1)],
         )
-        self._op_kernel_block = (16, 16, 1)
-        self._op_kernel_grid = [updiv(a, b) for a, b in zip(self.sinos_shape[1:][::-1], self._op_kernel_block[:-1])] + [
-            1
-        ]
-        self._op_kernel_args = [self.d_sino_profile, np.int32(self.n_x), np.int32(self.n_angles), np.int32(1)]
+        self._op_kernel_block = (16, 16, 4)
+        self._op_kernel_grid = [updiv(a, b) for a, b in zip(self.sinos_shape[::-1], self._op_kernel_block)]
+        self._op_kernel_args = [self.d_sino_profile, np.int32(self.n_x), np.int32(self.n_angles), np.int32(self.n_z)]
         self._op_kernel_kwargs = {
             "grid": self._op_kernel_grid,
             "block": self._op_kernel_block,
