@@ -30,11 +30,13 @@ class OpenCLSinoMult(SinoMult):
 
     def prepare_sino(self, sino):
         sino = self.opencl_processing.set_array("d_sino", sino)
-        self.halftomo_kernel(
+        ev = self.halftomo_kernel(
             self.opencl_processing.queue,
             sino,
             *self._halftomo_kernel_other_args,
             global_size=self._global_size,
             local_size=self._local_size,
         )
+        if self.opencl_processing.device_type == "cpu":
+            ev.wait()
         return sino
