@@ -236,7 +236,7 @@ class TestCone:
         err_median_profile = np.median(diff, axis=(-1, -2))
 
         assert np.max(err_max_profile) < 2e-3
-        assert np.max(err_median_profile) < 5e-6
+        assert np.max(err_median_profile) < 5.1e-6
 
     def test_reconstruction_horizontal_translations(self):
         n_z = n_y = n_x = 256
@@ -273,7 +273,7 @@ class TestCone:
         # Error tolerance has to be higher for these shifts.
         for shift_type, shifts, err_tol in [
             ("integer shifts", shifts_int, 5e-3),
-            ("float shifts", shifts_float, 1.4e-1),
+            ("float shifts", shifts_float, 1.5e-1),
         ]:
             cone_data_shifted = np.zeros_like(cone_data)
             [shift(cone_data[:, i, :], (0, shifts[i]), output=cone_data_shifted[:, i, :]) for i in range(n_a)]
@@ -281,8 +281,8 @@ class TestCone:
             # Reconstruct with horizontal shifts
             cone_reconstructor_with_correction = ConebeamReconstructor(
                 *reconstructor_args,
-                axis_correction=shifts,
                 **reconstructor_kwargs,
+                extra_options={"axis_correction": -shifts},
             )
 
             rec_with_correction = cone_reconstructor_with_correction.reconstruct(cone_data_shifted)
@@ -330,7 +330,6 @@ class TestCone:
 
             metric = lambda img: np.max(np.abs(clip_circle(img, radius=int(0.85 * 128))))
             error_profile = np.array([metric(rec[i] - rec_z[i]) for i in range(n_z)])
-            assert error_profile.max() < 3e-2, "Max error for padding=%s is too high" % padding_mode
 
             # import matplotlib.pyplot as plt
             # plt.figure()
@@ -338,6 +337,12 @@ class TestCone:
             # plt.legend([padding_mode])
             # plt.show()
 
+            assert error_profile.max() < 3.1e-2, "Max error for padding=%s is too high" % padding_mode
+            if padding_mode != "zeros":
+                assert not (np.allclose(rec[n_z // 2], rec_z[n_z // 2])), (
+                    "Reconstruction should be different when padding_mode=%s" % padding_mode
+                )
+
     def test_roi(self):
         n_z = n_y = n_x = 256
         n_a = 500
@@ -367,7 +372,7 @@ class TestCone:
         ref = cone_reconstructor_full.reconstruct(cone_data)
 
         # roi is in the form (start_x, end_x, start_y, end_y)
-        for roi in ((20, -20, 10, -10), (0, n_x, 0, n_y), (15, -15, 15, -15)):
+        for roi in ((20, -20, 10, -10), (0, n_x, 0, n_y), (50, -50, 15, -15)):
             # convert negative indices
             start_x, end_x, start_y, end_y = roi
             if start_y < 0:
