@@ -1,13 +1,18 @@
 import numpy as np
 import pytest
+from nabu.processing.fft_cuda import get_available_fft_implems
 from nabu.testutils import (
     get_big_data,
     __big_testdata_dir__,
-    compare_arrays,
     generate_tests_scenarios,
     __do_long_tests__,
 )
-from nabu.cuda.utils import __has_pycuda__, __has_cufft__, get_cuda_context
+from nabu.cuda.utils import __has_pycuda__, get_cuda_context
+
+__has_cufft__ = False
+if __has_pycuda__:
+    avail_fft = get_available_fft_implems()
+    __has_cufft__ = len(avail_fft) > 0
 
 __has_cuda_fbp__ = __has_cufft__ and __has_pycuda__
 if __has_cuda_fbp__:
@@ -43,7 +48,7 @@ def bootstrap(request):
 )
 @pytest.mark.usefixtures("bootstrap")
 class TestReconstructor:
-    @pytest.mark.skipif(not (__has_cuda_fbp__), reason="need pycuda and scikit-cuda")
+    @pytest.mark.skipif(not (__has_cuda_fbp__), reason="need pycuda and (scikit-cuda or vkfft)")
     @pytest.mark.parametrize("config", scenarios)
     def test_cuda_reconstructor(self, config):
         data = self.projs
