@@ -1,13 +1,14 @@
 import os
-from bisect import bisect_left
 import numpy as np
-from silx.io import get_data
 from silx.io.url import DataUrl
+from silx.io import get_data
 from tomoscan.esrf.scan.edfscan import EDFTomoScan
 from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
-from ..utils import check_supported
+
+from ..utils import check_supported, indices_to_slices
+from ..io.reader import EDFStackReader, NXDarksFlats, NXTomoReader
 from ..io.utils import get_compacted_dataslices
-from .utils import is_hdf5_extension, get_values_from_file
+from .utils import get_values_from_file, is_hdf5_extension
 from .logger import LoggerOrPrint
 
 from ..pipeline.utils import nabu_env_settings
@@ -51,7 +52,7 @@ class DatasetAnalyzer:
             "output_dir": None,
             "exclude_projections": None,
             "hdf5_entry": None,
-            "nx_version": 1.0,
+            # "nx_version": 1.0,
         }
         # --
         advanced_options.update(extra_options)
@@ -59,16 +60,18 @@ class DatasetAnalyzer:
 
     # pylint: disable=E1136
     def _get_excluded_projections(self):
-        self._ignore_projections_indices = None
-        self._need_rebuild_tomoscan_object_to_exclude_projections = False
         excluded_projs = self.extra_options["exclude_projections"]
+        self._ignore_projections = None
         if excluded_projs is None:
             return
-        if excluded_projs["type"] == "indices":
-            projs_idx = get_values_from_file(excluded_projs["file"], any_size=True).astype(np.int32).tolist()
-            self._ignore_projections_indices = projs_idx
-        else:
-            self._need_rebuild_tomoscan_object_to_exclude_projections = True
+
+        if excluded_projs["type"] == "angular_range":
+            excluded_projs["type"] = "range"  # compat with tomoscan #pylint: disable=E1137
+            values = excluded_projs["range"]
+        for ignore_kind, dtype in {"indices": np.int32, "angles": np.float32}.items():
+            if excluded_projs["type"] == ignore_kind:
+                values = get_values_from_file(excluded_projs["file"], any_size=True).astype(dtype).tolist()
+        self._ignore_projections = {"kind": excluded_projs["type"], "values": values}  # pylint: disable=E0606
 
     def _init_dataset_scan(self, **kwargs):
         if self._scanner is None:
@@ -83,40 +86,11 @@ class DatasetAnalyzer:
             kwargs["n_frames"] = 1
 
         self.dataset_scanner = self._scanner(  # pylint: disable=E1102
-            self.location, ignore_projections=self._ignore_projections_indices, **kwargs
+            self.location, ignore_projections=self._ignore_projections, **kwargs
         )
-        self.projections = self.dataset_scanner.projections
 
-        # ---
-        if self._need_rebuild_tomoscan_object_to_exclude_projections:
-            # pylint: disable=E1136
-            exclude_projs = self.extra_options["exclude_projections"]
-            rot_angles_deg = np.rad2deg(self.rotation_angles)
-            self._rotation_angles = None  # prevent caching
-            # tomoscan only supports ignore_projections=<list of integers>
-            # However this is cumbersome to use, it's more convenient to use angular range or list of angles
-            # But having angles instead of indices implies to already have information on current scan angular range
-            ignore_projections_indices = []
-            if exclude_projs["type"] == "angular_range":
-                exclude_angle_min, exclude_angle_max = exclude_projs["range"]
-                projections_indices = np.array(sorted(self.dataset_scanner.projections.keys()))
-                for proj_idx, angle in zip(projections_indices, rot_angles_deg):
-                    if exclude_angle_min <= angle and angle <= exclude_angle_max:
-                        ignore_projections_indices.append(proj_idx)
-            elif exclude_projs["type"] == "angles":
-                excluded_angles = get_values_from_file(exclude_projs["file"], any_size=True).astype(np.float32).tolist()
-                for excluded_angle in excluded_angles:
-                    proj_idx = bisect_left(rot_angles_deg, excluded_angle)
-                    if proj_idx < rot_angles_deg.size:
-                        ignore_projections_indices.append(proj_idx)
-            # Rebuild the dataset_scanner instance
-            self._ignore_projections_indices = ignore_projections_indices
-            self.dataset_scanner = self._scanner(  # pylint: disable=E1102
-                self.location, ignore_projections=self._ignore_projections_indices, **kwargs
-            )
-        # ---
-        if self._ignore_projections_indices is not None:
-            self.logger.info("Excluding projections: %s" % str(self._ignore_projections_indices))
+        if self._ignore_projections is not None:
+            self.logger.info("Excluding projections: %s" % str(self._ignore_projections))
 
         if nabu_env_settings.skip_tomoscan_checks:
             self.logger.warning(
@@ -124,9 +98,8 @@ class DatasetAnalyzer:
             )
             self.dataset_scanner.set_check_behavior(run_check=False, raise_error=False)
 
-        self.projections = self.dataset_scanner.projections
-        self.flats = self.dataset_scanner.flats
-        self.darks = self.dataset_scanner.darks
+        self.raw_flats = self.dataset_scanner.flats
+        self.raw_darks = self.dataset_scanner.darks
         self.n_angles = len(self.dataset_scanner.projections)
         self.radio_dims = (self.dataset_scanner.dim_1, self.dataset_scanner.dim_2)
         self._radio_dims_notbinned = self.radio_dims  # COMPAT
@@ -146,7 +119,10 @@ class DatasetAnalyzer:
         self._pixel_size = None
         self._distance = None
         self._flats_srcurrent = None
+        self._projections = None
         self._projections_srcurrent = None
+        self._reduced_flats = None
+        self._reduced_darks = None
 
     @property
     def energy(self):
@@ -223,15 +199,19 @@ class DatasetAnalyzer:
     def detector_tilt(self, tilt):
         self._detector_tilt = tilt
 
-    def _get_srcurrent(self, indices):
-        srcurrent = self.dataset_scanner.electric_current
-        if srcurrent is None or len(srcurrent) == 0:
-            return None
-        srcurrent_all = np.array(srcurrent)
-        if np.any(indices >= len(srcurrent_all)):
-            self.logger.error("Something wrong with SRCurrent: not enough values!")
-            return None
-        return srcurrent_all[indices].astype("f")
+    def _get_srcurrent(self, frame_type):
+        # To be implemented by inheriting class
+        return None
+
+    @property
+    def projections(self):
+        if self._projections is None:
+            self._projections = self.dataset_scanner.projections
+        return self._projections
+
+    @projections.setter
+    def projections(self, val):
+        raise ValueError
 
     @property
     def projections_srcurrent(self):
@@ -239,8 +219,7 @@ class DatasetAnalyzer:
         Return the synchrotron electric current for each projection.
         """
         if self._projections_srcurrent is None:
-            projections_indices = np.array(sorted(self.projections.keys()))
-            self._projections_srcurrent = self._get_srcurrent(projections_indices)
+            self._projections_srcurrent = self._get_srcurrent("radios")  # pylint: disable=E1128
         return self._projections_srcurrent
 
     @projections_srcurrent.setter
@@ -253,8 +232,7 @@ class DatasetAnalyzer:
         Return the synchrotron electric current for each flat image.
         """
         if self._flats_srcurrent is None:
-            flats_indices = np.array(sorted(self.flats.keys()))
-            self._flats_srcurrent = self._get_srcurrent(flats_indices)
+            self._flats_srcurrent = self._get_srcurrent("flats")  # pylint: disable=E1128
         return self._flats_srcurrent
 
     @flats_srcurrent.setter
@@ -268,6 +246,32 @@ class DatasetAnalyzer:
         if getattr(self, name, None) is None:
             raise ValueError(error_msg or str("No information on %s was found in the dataset" % name))
 
+    @property
+    def flats(self):
+        """
+        Return the REDUCED flat-field images. Either by reducing (median) the raw flats, or a user-defined reduced flats.
+        """
+        if self._reduced_flats is None:
+            self._reduced_flats = self.get_reduced_flats()
+        return self._reduced_flats
+
+    @flats.setter
+    def flats(self, val):
+        self._reduced_flats = val
+
+    @property
+    def darks(self):
+        """
+        Return the REDUCED flat-field images. Either by reducing (mean) the raw darks, or a user-defined reduced darks.
+        """
+        if self._reduced_darks is None:
+            self._reduced_darks = self.get_reduced_darks()
+        return self._reduced_darks
+
+    @darks.setter
+    def darks(self, val):
+        self._reduced_darks = val
+
 
 class EDFDatasetAnalyzer(DatasetAnalyzer):
     """
@@ -278,23 +282,7 @@ class EDFDatasetAnalyzer(DatasetAnalyzer):
     kind = "edf"
 
     def _finish_init(self):
-        self.remove_unused_radios()
-
-    def remove_unused_radios(self):
-        """
-        Remove "unused" radios.
-        This is used for legacy ESRF scans.
-        """
-        # Extraneous projections are assumed to be on the end
-        projs_indices = sorted(self.projections.keys())
-        used_radios_range = range(projs_indices[0], len(self.projections))
-        radios_not_used = []
-        for idx in self.projections.keys():
-            if idx not in used_radios_range:
-                radios_not_used.append(idx)
-        for idx in radios_not_used:
-            self.projections.pop(idx)
-        return radios_not_used
+        pass
 
     def _get_flats_darks(self):
         return
@@ -311,13 +299,34 @@ class EDFDatasetAnalyzer(DatasetAnalyzer):
         return None
 
     def _get_rotation_angles(self):
-        if self._rotation_angles is None:
-            scan_range = self.dataset_scanner.scan_range
-            if scan_range is not None:
-                fullturn = abs(scan_range - 360) < abs(scan_range - 180)
-                angles = np.linspace(0, scan_range, num=len(self.projections), endpoint=fullturn, dtype="f")
-                self._rotation_angles = np.deg2rad(angles)
-        return self._rotation_angles
+        return np.deg2rad(self.dataset_scanner.rotation_angle())
+
+    def get_reduced_flats(self, **reader_kwargs):
+        if self.raw_flats in [None, {}]:
+            raise FileNotFoundError("No reduced flat ('refHST') found in %s" % self.location)
+        # A few notes:
+        # (1) In principle we could do the reduction (mean/median) from raw frames (ref_xxxx_yyyy)
+        #   but for legacy datasets it's always already done (by fasttomo3), and EDF support is supposed to be dropped on our side
+        # (2) We use EDFStackReader class to handle the possible additional data modifications
+        #  (eg. subsampling, binning, distortion correction...)
+        # (3) The following spawns one reader instance per file, which is not elegant,
+        #   but in principle there are typically 1-2 reduced flats in a scan
+        readers = {k: EDFStackReader([self.raw_flats[k].file_path()], **reader_kwargs) for k in self.raw_flats.keys()}
+        return {k: readers[k].load_data()[0] for k in self.raw_flats.keys()}
+
+    def get_reduced_darks(self, **reader_kwargs):
+        # See notes in get_reduced_flats() above
+        if self.raw_darks in [None, {}]:
+            raise FileNotFoundError("No reduced dark ('darkend.edf' or 'dark.edf') found in %s" % self.location)
+        readers = {k: EDFStackReader([self.raw_darks[k].file_path()], **reader_kwargs) for k in self.raw_darks.keys()}
+        return {k: readers[k].load_data()[0] for k in self.raw_darks.keys()}
+
+    @property
+    def files(self):
+        return sorted([u.file_path() for u in self.dataset_scanner.projections.values()])
+
+    def get_reader(self, **kwargs):
+        return EDFStackReader(self.files, **kwargs)
 
 
 class HDF5DatasetAnalyzer(DatasetAnalyzer):
@@ -326,7 +335,10 @@ class HDF5DatasetAnalyzer(DatasetAnalyzer):
     """
 
     _scanner = NXtomoScan
-    kind = "hdf5"
+    kind = "nx"
+    # We could import the 1000+ LoC  nxtomo.nxobject.nxdetector.ImageKey... or we can do this
+    _image_key_value = {"flats": 1, "darks": 2, "radios": 0}
+    #
 
     @property
     def z_translation(self):
@@ -353,10 +365,10 @@ class HDF5DatasetAnalyzer(DatasetAnalyzer):
     def _get_dataset_hdf5_url(self):
         if len(self.projections) > 0:
             frames_to_take = self.projections
-        elif len(self.flats) > 0:
-            frames_to_take = self.flats
-        elif len(self.darks) > 0:
-            frames_to_take = self.darks
+        elif len(self.raw_flats) > 0:
+            frames_to_take = self.raw_flats
+        elif len(self.raw_darks) > 0:
+            frames_to_take = self.raw_darks
         else:
             raise ValueError("No projections, no flats and no darks ?!")
         first_proj_idx = sorted(frames_to_take.keys())[0]
@@ -397,8 +409,13 @@ class HDF5DatasetAnalyzer(DatasetAnalyzer):
         slices: list of slice
             A list where each item is a slice.
         """
-        check_supported(what, ["projections", "flats", "darks"], "image type")
-        images = getattr(self, what)  # dict
+        name_to_attr = {
+            "projections": self.projections,
+            "flats": self.raw_flats,
+            "darks": self.raw_darks,
+        }
+        check_supported(what, name_to_attr.keys(), "image type")
+        images = name_to_attr[what]  # dict
         # we can't directly use set() on slice() object (unhashable). Use tuples
         slices = set()
         for du in get_compacted_dataslices(images).values():
@@ -410,6 +427,39 @@ class HDF5DatasetAnalyzer(DatasetAnalyzer):
         slices_list = [slice(item[0], item[1]) if item is not None else None for item in list(slices)]
         return slices_list
 
+    def _select_according_to_frame_type(self, data, frame_type):
+        if data is None:
+            return None
+        return data[self.dataset_scanner.image_key_control == self._image_key_value[frame_type]]
+
+    def get_reduced_flats(self, method="median", force_reload=False, **reader_kwargs):
+        dkrf_reader = NXDarksFlats(
+            self.dataset_hdf5_url.file_path(), data_path=self.dataset_hdf5_url.data_path(), **reader_kwargs
+        )
+        return dkrf_reader.get_reduced_flats(method=method, force_reload=force_reload, as_dict=True)
+
+    def get_reduced_darks(self, method="mean", force_reload=False, **reader_kwargs):
+        dkrf_reader = NXDarksFlats(
+            self.dataset_hdf5_url.file_path(), data_path=self.dataset_hdf5_url.data_path(), **reader_kwargs
+        )
+        return dkrf_reader.get_reduced_darks(method=method, force_reload=force_reload, as_dict=True)
+
+    def _get_srcurrent(self, frame_type):
+        return self._select_according_to_frame_type(self.dataset_scanner.electric_current, frame_type)
+
+    def frames_slices(self, frame_type):
+        """
+        Return a list of slice objects corresponding to the data corresponding to "frame_type".
+        For example, if the dataset flats are located at indices [1, 2, ..., 99], then
+        frame_slices("flats") will return [slice(0, 100)].
+        """
+        return indices_to_slices(
+            np.where(self.dataset_scanner.image_key_control == self._image_key_value[frame_type])[0]
+        )
+
+    def get_reader(self, **kwargs):
+        return NXTomoReader(self.dataset_hdf5_url.file_path(), data_path=self.dataset_hdf5_url.data_path(), **kwargs)
+
 
 def analyze_dataset(dataset_path, extra_options=None, logger=None):
     if not (os.path.isdir(dataset_path)):
