@@ -96,9 +96,9 @@ class TestNXFlatField:
         for s in self.params["darks_pos"]:
             expected_darks[s.start] = self._reduction_func["darks"](data_volume[s.start : s.stop], axis=0)
 
-        flats = {idx: get_data(dataset_info.flats[idx]) for idx in dataset_info.flats.keys()}
+        flats = dataset_info.flats
         for idx in flats.keys():
             assert np.allclose(flats[idx], expected_flats[idx])
-        darks = {idx: get_data(dataset_info.darks[idx]) for idx in dataset_info.darks.keys()}
+        darks = dataset_info.darks
         for idx in darks.keys():
             assert np.allclose(darks[idx], expected_darks[idx])
