@@ -39,7 +39,9 @@ def nabu_test():
     args = sys.argv[1:]
     modules_to_test, extra_args = get_modules_to_test(args)
     if len(modules_to_test) == 0:
-        modules_to_test = [get_folder_path()]
+        modules_to_test = [
+            nabu_folder,
+        ]
     pytest_args = extra_args + modules_to_test
     return pytest.main(pytest_args)
 
