@@ -4,9 +4,11 @@
 This file is a "GPU" (through cupy) implementation of "remove_all_stripe".
 The original method is implemented by Nghia Vo in the algotom project: https://github.com/algotom/algotom/blob/master/algotom/prep/removal.py
 The implementation using cupy is done by Viktor Nikitin in the tomocupy project: https://github.com/tomography/tomocupy/blame/main/src/tomocupy/remove_stripe.py
-License follows.
+then moved to https://github.com/tomography/tomocupy/blob/main/src/tomocupy/processing/remove_stripe.py
 
 For now we can't rely on off-the-shelf tomocupy as it's not packaged in pypi, and compilation is quite tedious.
+
+License follows.
 """
 
 # *************************************************************************** #
@@ -47,6 +49,7 @@ For now we can't rely on off-the-shelf tomocupy as it's not packaged in pypi, an
 # *************************************************************************** #
 
 try:
+    import pycuda.gpuarray as garray
     import cupy as cp
     import pywt
     from cupyx.scipy.ndimage import median_filter
@@ -559,28 +562,59 @@ def remove_all_stripe(tomo,
     return tomo
 
 
-from ..cuda.utils import pycuda_to_cupy
-def remove_all_stripe_pycuda(radios, device_id=0, **kwargs):
+def remove_all_stripe_sinos(sinos, snr=3, la_size=61, sm_size=21, dim=1):
     """
-    Nabu interface to "remove_all_stripe". In-place!
+    Same as remove_all_stripe(), but acting on sinograms
+    """
+    n_sinos, n_a, n_x = sinos.shape
+    matindex = _create_matindex(n_x, n_a)
+    for m in range(n_sinos):
+        sino = sinos[m]
+        sino = _rs_dead(sino, snr, la_size, matindex)
+        sino = _rs_sort(sino, sm_size, matindex, dim)
+        sinos[m] = sino
+    return sinos
+
+
+from ..cuda.utils import pycuda_to_cupy
+def remove_all_stripe_pycuda(array, layout="radios", device_id=0, **kwargs):
+    """
+    Nabu interface to tomocupy "remove_all_stripe".
+    Processing is done in-place to save memory, meaning that the content of "array" will be overwritten.
 
     Parameters
     ----------
-    radios: pycuda.GPUArray
-        Stack of radios in the shape (n_angles, n_y, n_x)
-        so that sinogram number i is radios[:, i, :]
+    array: pycuda.GPUArray
+        Stack of radios in the shape (n_angles, n_y, n_x), if layout == "radios"
+        Stack of sinos in the shape (n_y, n_angles, n_x), if layout == "sinos".
+
 
     Other Parameters
     ----------------
     See parameters of 'remove_all_stripe
     """
 
+    # Init cupy. Nabu currently does not use cupy, with exception of this module,
+    # so the initialization has to be done here.
     if getattr(remove_all_stripe, "_cupy_init", False) is False:
         from cupy import cuda
         cuda.Device(device_id).use()
         setattr(remove_all_stripe, "_cupy_init", True)
 
-    cupy_radios = pycuda_to_cupy(radios) # no memory copy, the internal pointer is passed to pycuda
-    remove_all_stripe(cupy_radios, **kwargs)
-    return radios
+    # remove_all_stripe() in tomocupy expects a 3D array to build the "matindex" data structure.
+    # The usage of this "matindex" array is not clear since this method is supposed to act on individual sinograms.
+    # To avoid memory duplication, we use fake 3D array, i.e, we pass a series of (1, n_a, n_x) sinograms
 
+    if layout == "radios":
+        sinos = array.transpose(axes=(1, 0, 2)) # no copy
+    else:
+        sinos = array
+    # is_contiguous = sinos.flags.c_contigious
+    n_sinos, n_a, n_x = sinos.shape
+    sinos_tmp = garray.zeros((1, n_a, n_x), dtype="f")
+    for i in range(n_sinos):
+        sinos_tmp[0] = sinos[i]
+        cupy_sinos = pycuda_to_cupy(sinos_tmp) # no memory copy, the internal pointer is passed to pycuda
+        remove_all_stripe_sinos(cupy_sinos, **kwargs)
+        sinos[i] = sinos_tmp[0]
+    return array
