@@ -1,6 +1,6 @@
+from fnmatch import fnmatch
 from functools import partial
 import os
-import sys
 from functools import partial, lru_cache
 from itertools import product
 import warnings
@@ -50,7 +50,7 @@ def convert_index(idx, idx_max, default_val):
 
 
 def get_folder_path(foldername=""):
-    _file_dir = os.path.dirname(os.path.realpath(__file__))
+    _file_dir = os.path.dirname(os.path.abspath(__file__))
     package_dir = _file_dir
     return os.path.join(package_dir, foldername)
 
@@ -74,6 +74,99 @@ def get_resource_file(filename, subfolder=None):
     return os.path.join(abspath, filename)
 
 
+def indices_to_slices(indices):
+    """
+    From a series of integer indices, return corresponding slice() objects.
+
+    Parameters
+    ----------
+    indices: collection of sorted unique integers
+        Arrays indices
+
+    Examples
+    --------
+    slices_from_indices([0, 1, 2, 3]) returns [slice(0, 4)]
+    slices_from_indices([8, 9, 10, 14, 15, 16]) returns [slice(8, 11), slice(15, 17)]
+    """
+    jumps = np.where(np.diff(indices) > 1)[0]
+    if len(jumps) == 0:
+        return [slice(indices[0], indices[-1] + 1)]
+    jumps = np.hstack([-1, jumps, len(indices) - 1])
+    slices = []
+    for i in range(len(jumps) - 1):
+        slices.append(slice(indices[jumps[i] + 1], indices[jumps[i + 1]] + 1))
+    return slices
+
+
+def merge_slices(slice1, slice2):
+    """
+    Merge two slicing operations in one.
+
+    Examples
+    --------
+    array = numpy.arange(200)
+    array[slice(133, 412, 2)][slice(31, 35, 2)] # gives [195 199]
+    array[merge_slices(slice(133, 412, 2), slice(31, 35, 2))]  # gives [195 199]
+    """
+    step1 = slice1.step or 1
+    step2 = slice2.step or 1
+    step = step1 * step2
+    if step == 1:
+        step = None
+
+    start = slice1.start + step1 * (slice2.start or 0)
+    if slice2.stop is None:
+        stop = slice1.stop
+    else:
+        stop = min(slice1.stop, slice1.start + step1 * slice2.stop)
+    return slice(start, stop, step)
+
+
+def compacted_views(slices_):
+    """
+    From a list of slice objects, returns the slice objects corresponding to a compact view.
+
+    If "array" is obtained with
+        array = np.hstack([big_array[slice1], big_array[slice2]])
+    Then, compacted_views([slice1, slice2]) returns [slice3, slice4] where
+      - slice3 are the indices, in 'array', corresponding to indices of slice1 in 'big_array'
+      - slice4 are the indices, in 'array', corresponding to indices of slice2 in 'big_array'
+
+    Example
+    -------
+    compacted_views([slice(1, 26), slice(526, 551)]) gives [slice(0, 25), slice(25, 50)]
+
+    """
+    prev_start = 0
+    r = []
+    for s in slices_:
+        start = prev_start
+        stop = start + (s.stop - s.start)
+        r.append(slice(start, stop))
+        prev_start = stop
+    return r
+
+
+def get_size_from_sliced_dimension(length, slice_):
+    """
+    From a given array size, returns the size of the array once it is accessed using a slice.
+
+    Examples
+    --------
+    If data.shape = (3500, 2160, 2560)
+    get_size_from_sliced_dimension(data.shape[0], None) returns 3500
+    get_size_from_sliced_dimension(data.shape[0], slice(100, 200)) returns 100
+    """
+    return np.arange(length)[slice_].size
+
+
+def get_shape_from_sliced_dims(shape, slices_):
+    """
+    Same as get_size_from_sliced_dimension() but in 3D
+    """
+    return tuple(get_size_from_sliced_dimension(length, slice_) for length, slice_ in zip(shape, slices_))
+
+
 def get_available_threads():
     try:
         n_threads = len(os.sched_getaffinity(0))
@@ -82,6 +175,18 @@ def get_available_threads():
     return n_threads
 
 
+def list_match_queries(available, queries):
+    """
+    Given a list of strings, return all items matching any of one elements of "queries"
+    """
+    matches = []
+    for a in available:
+        for q in queries:
+            if fnmatch(a, q):
+                matches.append(a)
+    return matches
+
+
 def is_writeable(location):
     """
     Return True if a file/location is writeable.
@@ -93,6 +198,12 @@ def is_int(num, eps=1e-7):
     return abs(num - int(num)) < eps
 
 
+def is_scalar(stuff):
+    if isinstance(stuff, str):
+        return False
+    return np.isscalar(stuff)
+
+
 def _sizeof(Type):
     """
     return the size (in bytes) of a scalar type, like the C behavior
@@ -100,6 +211,22 @@ def _sizeof(Type):
     return np.dtype(Type).itemsize
 
 
+class _Default_format(dict):
+    """
+    https://docs.python.org/3/library/stdtypes.html
+    """
+
+    def __missing__(self, key):
+        return key
+
+
+def safe_format(str_, **kwargs):
+    """
+    Alternative to str.format(), but does not throw a KeyError when fields are missing.
+    """
+    return str_.format_map(_Default_format(**kwargs))
+
+
 def get_ftype(url):
     """
     return supposed filetype of an url
@@ -294,6 +421,11 @@ def partition_dict(dict_, n_partitions):
     return res
 
 
+def first_dict_item(dict_):
+    keys = sorted(list(dict_.keys()))
+    return dict_[keys[0]]
+
+
 def subsample_dict(dic, subsampling_factor):
     """
     Subsample a dict where keys are integers.
@@ -559,64 +691,6 @@ def convert_str_to_tuple(input_str: str, none_if_empty: bool = False):
         return tuple(elmts)
 
 
-class Progress:
-    """Simple interface for defining advancement on a 100 percentage base"""
-
-    def __init__(self, name: str):
-        self._name = name
-        self.set_name(name)
-
-    def set_name(self, name):
-        self._name = name
-        self.reset()
-
-    def reset(self, max_=None):
-        """
-        reset the advancement to n and max advancement to max_
-        :param int max_:
-        """
-        self._n_processed = 0
-        self._max_processed = max_
-
-    def start_process(self) -> None:
-        self.set_advancement(0)
-
-    def set_advancement(self, value: int) -> None:
-        """
-
-        :param int value: set advancement to value
-        """
-        length = 20  # modify this to change the length
-        block = int(round(length * value / 100))
-        blocks_str = "#" * block + "-" * (length - block)
-        msg = "\r{0}: [{1}] {2}%".format(self._name, blocks_str, round(value, 2))
-        if value >= 100:
-            msg += " DONE\r\n"
-        sys.stdout.write(msg)
-        sys.stdout.flush()
-
-    def end_process(self) -> None:
-        """Set advancement to 100 %"""
-        self.set_advancement(100)
-
-    def set_max_advancement(self, n: int) -> None:
-        """
-
-        :param int n: number of steps contained by the advancement. When
-        advancement reach this value, advancement will be 100 %
-        """
-        self._max_processed = n
-
-    def increase_advancement(self, i: int = 1) -> None:
-        """
-
-        :param int i: increase the advancement of n step
-        """
-        self._n_processed += i
-        advancement = int(float(self._n_processed / self._max_processed) * 100)
-        self.set_advancement(advancement)
-
-
 def concatenate_dict(dict_1, dict_2) -> dict:
     """update dict which has dict as values. And we want concatenate those values to"""
     res = dict_1.copy()
@@ -658,12 +732,14 @@ def generate_coords(img_shp, center=None):
     return R, C
 
 
-def clip_circle(img, center=None, radius=None):
+def clip_circle(img, center=None, radius=None, out_value=0):
     R, C = generate_coords(img.shape, center)
     if radius is None:
         radius = R.shape[-1] // 2
     M = R**2 + C**2
     res = np.zeros_like(img)
+    if out_value != 0:
+        res.fill(out_value)
     res[M < radius**2] = img[M < radius**2]
     return res
 
@@ -821,3 +897,24 @@ def docstring(origin):
         If the origin class has not method n case the
     """
     return partial(_docstring, origin=origin)
+
+
+from warnings import catch_warnings
+
+# FIX for python < 3.11
+# catch_warnings() does not have "action=XX" kwarg for python < 3.11
+from sys import version_info
+
+if version_info.major == 3 and version_info.minor < 11:
+
+    def dummy(*args, **kwargs):
+        pass
+
+    catch_warnings_old = catch_warnings
+
+    def catch_warnings(*args, **kwargs):  # pylint: disable=E0102
+        action = kwargs.pop("action", None)
+        return catch_warnings_old(record=(dummy if action == "ignore" else False))
+
+
+# ---
